/*
 * $Id: JPythonEngine.java,v 1.1.1.1 2000/06/20 19:19:11 samruby Exp $
 *
 * $Log: JPythonEngine.java,v $
 * Revision 1.1.1.1  2000/06/20 19:19:11  samruby
 * Initial import
 *
 * Revision 1.6  2000/04/13 18:30:59  rubys
 * unwrap Java instances
 *
 * Revision 1.5  1999/10/04 15:12:54  duftler
 * Needed to declare initial beans in initialize(...).
 *
 * Revision 1.4  1999/10/01 17:48:05  sanjiva
 * added declare/undeclare bean stuff
 *
 * Revision 1.3  1999/09/20 17:29:09  sanjiva
 * done with JPython
 *
 * Revision 1.2  1999/09/17 23:49:37  rubys
 * Build errors due to CVS tree reorg
 *
 * Revision 1.1  1999/08/04 19:04:52  sanjiva
 * mods
 *
 *
 */

package com.ibm.bsf.engines.jpython;

import java.util.Vector;

import org.python.util.*;
import org.python.core.*;

import com.ibm.bsf.*;
import com.ibm.bsf.util.BSFEngineImpl;
import com.ibm.bsf.util.BSFFunctions;

/**
 * This is the interface to JPython (http://www.jpython.org/) from BSF.
 *
 * @version  $Revision: 1.1.1.1 $
 * @author   Sanjiva Weerawarana
 */

public class JPythonEngine extends BSFEngineImpl {
  PythonInterpreter interp;

  /**
   * Initialize the engine.
   */
  public void initialize (BSFManager mgr, String lang,
                          Vector declaredBeans) throws BSFException {
    super.initialize (mgr, lang, declaredBeans);

    // create an interpreter
    interp = new PythonInterpreter ();

    // register the mgr with object name "bsf"
    interp.set ("bsf", new BSFFunctions (mgr, this));

    int size = declaredBeans.size ();
    for (int i = 0; i < size; i++) {
      declareBean ((BSFDeclaredBean) declaredBeans.elementAt (i));
    }
  }

  /**
   * Evaluate an expression.
   */
  public Object eval (String source, int lineNo, int columnNo, 
		      Object script) throws BSFException {
    try {
      Object result = interp.eval (script.toString ());
      if (result != null && result instanceof PyJavaInstance)
        result = ((PyJavaInstance)result).__tojava__(Object.class);
      return result;
    } catch (PyException e) {
      e.printStackTrace ();
      throw new BSFException (BSFException.REASON_EXECUTION_ERROR,
			      "exception from JPython: " + e, e);
    }
  }

  /**
   * Execute a script. 
   */
  public void exec (String source, int lineNo, int columnNo,
		    Object script) throws BSFException {
    try {
      interp.exec (script.toString ());
    } catch (PyException e) {
      e.printStackTrace ();
      throw new BSFException (BSFException.REASON_EXECUTION_ERROR,
			      "exception from JPython: " + e, e);
    }
  }

  /**
   * call the named method of the given object.
   */
  public Object call (Object object, String method, Object[] args) 
                                                        throws BSFException {
    StringBuffer sb = new StringBuffer (300);

    if (object != null) {
      sb.append (object.toString());
      sb.append (".");
    }
    sb.append (method);
    sb.append ("(");
    if (args != null) {
      for (int i = 0; i < args.length; i++) {
	sb.append (args[i].toString ());
        if (i < args.length-1) {
          sb.append (",");
        }
      }
    }
    sb.append (")");
    String toRun = sb.toString ();

    // try to run this as an expr first and if not as a stmt 
    try {
      return eval ("<jpython-call>", 0, 0, toRun);
    } catch (BSFException e) {
      exec ("<jpython-call>", 0, 0, toRun);
      return null;
    }
  }

  /**
   * Declare a bean
   */
  public void declareBean (BSFDeclaredBean bean) throws BSFException {
    interp.set (bean.name, bean.bean);
  }

  /**
   * Undeclare a previously declared bean.
   */
  public void undeclareBean (BSFDeclaredBean bean) throws BSFException {
    interp.set (bean.name, null);
  }
}
