--##############################################################
--# The BSFManager
--##############################################################
::CLASS BSFManager PUBLIC

--**************************************************************
--* init, does nothing at the moment
--**************************************************************
::METHOD init

--**************************************************************
--* class method to import a Java Bean class into Object Rexx
--**************************************************************
::METHOD import
  USE ARG classname, java_path 

--define new class as a subclass to .BSFClass
  new_class = .BSFClass~subclass(classname)   

--create code for the INIT-method of the new class
  method_txt = 'self~init:super(''' || java_path || ''', ARG(1,''A''))'

--add code to new class under the method-name INIT
  new_class~define('init', method_txt)

--make new class available via local environment
  make_class = '.local~' || classname || '=new_class'
  interpret make_class

--**************************************************************
--* runs a loop, that polls BSF for events
--**************************************************************
::METHOD loop
  do forever
    interpret PollBSFEvents()
  end

--**************************************************************
--* exits BSF and Object REXX
--**************************************************************
::METHOD exit
  CallBSF('exit')

--##############################################################
--# The super-class for all 'bean'-classes in ObjectREXX
--##############################################################
::CLASS BSFClass PUBLIC

--**************************************************************
--* counter, used to name BSF-objects
--**************************************************************
::METHOD ObjectCount CLASS ATTRIBUTE 

::METHOD init CLASS
  USE ARG args
  self~ObjectCount = 0
  self~init:super

::METHOD new CLASS
  EXPOSE ObjectCount
  ObjectCount=ObjectCount + 1
  FORWARD CLASS (super)

--**************************************************************
--* inits a new BSF-object
--**************************************************************
::METHOD init
  USE ARG bean, args

--create a unique bean name, using DefaultName and a number
  objName = self~DefaultName
  PARSE VAR objName a name
  self~ObjectName = name || self~class~ObjectCount

--register bean with BSF
  MethodCall = 'CallBSF(registerBean, ''' || self~objectname || ''', ''' || bean 

--call constructor method with given arguments
  if args~items>0 THEN DO
    MethodCall = MethodCall || ''', '''
    DO i=1 TO args~items-1
      MethodCall = MethodCall || args[i] || ''', ''' 
    END
    MethodCall = MethodCall || args[args~items]
  END
  MethodCall = MethodCall || ''')'
  interpret MethodCall

--**************************************************************
--* uninit a BSF-object
--**************************************************************
::METHOD uninit
  MethodCall = 'CallBSF(unregisterBean, ''' || self~objectname || ''')'
  interpret MethodCall

--**************************************************************
--* add an event listener to a BSF-object
--**************************************************************
::METHOD event
  USE ARG a1, a2, a3
  MethodCall = 'CallBSF(addEventListener, ''' || self~objectname || ''', ''' || a1 || ''', ''' || a2 || ''', ''' || a3 || ''')'
  interpret MethodCall

--**************************************************************
--* call a BSF-object's method with given arguments 
--* and return a result
--**************************************************************
::METHOD unknown
  USE ARG name, args
  result = ''
  MethodCall = 'result = CallBSF(callFunction, ''' || self~objectname || ''', ''' || name
  if args~items>0 THEN DO
    MethodCall = MethodCall || ''', '''
    DO i=1 TO args~items-1
      MethodCall = MethodCall || args[i] || ''', ''' 
    END
    MethodCall = MethodCall || args[args~items]
  END
  MethodCall = MethodCall || ''')'
  interpret MethodCall
  return result