#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <jni.h>
#include "com_ibm_bsf_engines_objrexx_JavaObjRexxSAA.h"
#define INCL_RXSHV
#define INCL_RXFUNC
#include <rexx.h>

#ifdef _MSC_VER
# pragma warning(disable:4100)
#endif

#define DLLNAME "JaObjReSAA"

RexxFunctionHandler PollBSFEvents;
RexxFunctionHandler CallBSF;

JNIEnv *GlobEnv;	//used by CallBSF to call back to Java
jobject GlobObj;	//used by CallBSF to call back to Java

class List {
	struct Element {
		char *Value;
		Element *Next;
	};
	Element *First;
	Element *Last;
public:
	List() {
		First = 0;
		Last = 0;
	}

	boolean Empty() {
		if(First==0) return true;
		else return false;
	}

	void Add(char *str) {		
		Element *l = new Element;
		l->Value = new char[strlen(str)];
		strcpy(l->Value, str);
		l->Next = 0;
		if(First==0) {
			First = l;
			Last = l;
		} else {
			Last->Next = l;
			Last = l;
		}
	}

	char *Read() {
		return First->Value;		
	}

	void Remove() {
		Element *l = First;
		First = First->Next;
		delete l;
	}

	~List() {
		while(First) {
			Remove();
		}
	}
};

List *EventList;

/************************************************************
 * 'PollBSFEvents' is called from Rexx, to see if any new
 * events have occured
 ************************************************************/ 
APIRET APIENTRY 
PollBSFEvents(PSZ name,
		  ULONG argc,
		  PRXSTRING argv,
		  PSZ queuename,
		  PRXSTRING retstr) {
	if(EventList->Empty()) {
		retstr->strptr = new char[0];
		retstr->strlength = 0;
	} else {
		retstr->strptr = EventList->Read();		
		retstr->strlength = strlen(retstr->strptr);
		EventList->Remove();
	}		
	return 0;	
}

/************************************************************
 * 'CallBSF' calls back to the Java method 'CallBSF'
 ************************************************************/ 
APIRET APIENTRY 
CallBSF(PSZ name,
		  ULONG argc,
		  PRXSTRING argv,
		  PSZ queuename,
		  PRXSTRING retstr) {
	//get Java method 'CallBSF'
	jclass cls = (*GlobEnv).GetObjectClass(GlobObj);
    jmethodID mid = (*GlobEnv).GetMethodID(cls, "CallBSF", 
										   "([Ljava/lang/String;)Ljava/lang/String;");
    if (mid == 0) return -1;

	//convert arguments array to Java format
	jobjectArray arr;	
    arr = (*GlobEnv).NewObjectArray(argc, (*GlobEnv).FindClass("java/lang/String"),
									(*GlobEnv).NewStringUTF(""));
	for(ULONG i=0; i<argc; i++) {
        (*GlobEnv).SetObjectArrayElement(arr, i, 
										 (*GlobEnv).NewStringUTF(argv[i].strptr));
	}

	//call Java method and create Rexx return value
    jstring ReturnValue = (jstring)(*GlobEnv).CallObjectMethod(GlobObj, mid, arr);	
	if(ReturnValue!=NULL) {
		const char *str = GlobEnv->GetStringUTFChars(ReturnValue, 0);		
		retstr->strptr = new char[strlen(str)];
		strcpy(retstr->strptr, str);
		retstr->strlength = strlen(str);		
	} else {
		retstr->strptr = new char[0];
		retstr->strlength = 0;
	}	
	return 0;
}


/************************************************************
 * 'RegisterBSF' registers the 'CallBSF' method with the
 * Rexx interpreter
 ************************************************************/ 
JNIEXPORT jlong JNICALL 
Java_com_ibm_bsf_engines_objrexx_JavaObjRexxSAA_RegisterBSF(JNIEnv *env,
													  jobject obj) {
	LONG ReturnValue;
	ReturnValue = RexxRegisterFunctionDll("CallBSF", "JaObjReSAA", "CallBSF");	
	ReturnValue = RexxRegisterFunctionDll("PollBSFEvents", "JaObjReSAA", "PollBSFEvents");	
    return ReturnValue;
}

/************************************************************
 * 'Start' runs a piece of Rexx script
 ************************************************************/ 
JNIEXPORT jlong JNICALL 
Java_com_ibm_bsf_engines_objrexx_JavaObjRexxSAA_Start(JNIEnv *env, 
												jobject obj, 
												jstring source) {	
	//set global variables for callbacks to Java
	GlobEnv = env;
	GlobObj = obj;

	//create empty event list
	EventList = new List;

	//convert Java string containing script to Rexx string
	const char *str = env->GetStringUTFChars(source, 0);    	
	char *sourceStr;
	sourceStr = new char[strlen(str)];
	strcpy(sourceStr, str);		
	PRXSTRING Instore;
	RXSTRING RXsourceStr[2];
	MAKERXSTRING(RXsourceStr[0], sourceStr, strlen(sourceStr));
	MAKERXSTRING(RXsourceStr[1], NULL, NULL);

	//start Rexx
	SHORT ReturnCode;
	LONG ReturnValue;
	char ResultBuffer[250];
	RXSTRING Result;
	MAKERXSTRING(Result, ResultBuffer, sizeof(ResultBuffer));
	Instore = &RXsourceStr[0];
	PSZ	ProgramName = "";	
	ReturnValue = RexxStart(
			0,				//LONG		ArgCount
			NULL,			//PRXSTRING ArgList
			ProgramName,	//PSZ		ProgramName
			Instore,		//PRXSTRING	Instore
			NULL,			//PSZ		EnvName
			RXSUBROUTINE,	//LONG		CallType
			NULL,			//PRXSYSEXIT Exits
			&ReturnCode,	//PUSHORT	ReturnCode
			&Result			//PRXSTRING Result
			);	
	GlobalFree(&RXsourceStr[1]);	
	env->ReleaseStringUTFChars(source, str);
	return ReturnValue;	
}

/************************************************************
 * 'SetEventScript' saves an event script in the global 
 * variable EventScript
 ************************************************************/ 
JNIEXPORT void JNICALL 
Java_com_ibm_bsf_engines_objrexx_JavaObjRexxSAA_SetEventScript(JNIEnv *env, 
														 jobject obj, 
														 jstring source) {		
	const char *str = env->GetStringUTFChars(source, 0);
	char *EventScript = new char[strlen(str)];
	strcpy(EventScript, str);
	EventList->Add(EventScript);	
}
