package com.ibm.bsf.engines.objrexx;

import java.util.*;
import java.io.*;
import com.ibm.bsf.*;
import com.ibm.bsf.util.BSFEngineImpl;
import com.ibm.bsf.engines.objrexx.JavaObjRexxSAA;

/************************************************************
 * This is the interface to use (Regina) Rexx with the BSF
 ************************************************************/

public class ObjRexxEngine extends BSFEngineImpl {          
    
    private JavaObjRexxSAA ObjRexx;  //the link to the Rexx interpreter
    private String mainScript; //used to store the script, needed 
                               //for calls to Rexx functions
    
    /************************************************************
     * 'initialize' creates a link to the Rexx interpreter and 
     * registers the 'callBSF' function with Rexx
     ************************************************************/
    public void initialize (BSFManager mgr, String lang,
                            Vector declaredBeans) throws BSFException {
        super.initialize (mgr, lang, declaredBeans);        
        ObjRexx = new JavaObjRexxSAA(mgr, this);        
        ObjRexx.RegisterBSF();        
    }
    
    /************************************************************
     * 'eval' runs a piece of Rexx script 
     * if the script is marked as beeing an '<event-script>'
     * the main script is added, so that any subcommands defined
     * there can be run appropriately
     ************************************************************/
    public Object eval (String source, int lineNo, int columnNo, 
                        Object oscript) throws BSFException {                
        String script = oscript.toString ();
        if(source.equals("<event-script>")){                        
            ObjRexx.SetEventScript(script);
        } else {
            ObjRexx.Start(script);
        }        
        return null;
    }
    
    /************************************************************
     * 'call' has to be present, because it's part of the 
     * BSFEngine interface, however, there is no way to implement
     * calls to Objects in a non-oo-language
     ************************************************************/
    public Object call (Object object, String method, Object[] args)
                        throws BSFException {
        return null;
    }
}