package com.ibm.bsf.engines.tcl;

import java.util.*;
import java.io.*;
import com.ibm.bsf.*;
import com.ibm.bsf.util.BSFEngineImpl;
import tcl.lang.* ;

public class TclEngine extends BSFEngineImpl {
  // tcl interpreter, saved as an object and passed into the eval/call
  // methods to cast into the right type
  tcl.lang.Interp interp;

  /*
   * initialize the engine
   */

  public void initialize (BSFManager mgr, String lang,
                          Vector declaredBeans) throws BSFException {
      super.initialize (mgr, lang, declaredBeans);
      interp = new Interp();
      for (int i = 0; i < declaredBeans.size() ; i++) 
          declareBean( (BSFDeclaredBean) declaredBeans.elementAt(i) );
  }

  public void declareBean(BSFDeclaredBean bean) throws BSFException {
      try {
          TclObject tclBean = ReflectObject.newInstance( interp, bean.type,
                                                         bean.bean );
          interp.setVar( bean.name, tclBean, 0 );
      } catch( Exception e ) {
	  throw new BSFException (BSFException.REASON_EXECUTION_ERROR,
	                          "error while undeclaring bean '" + bean.name +
				  "': " + e.toString(), e);
      }
  }

  public void undeclareBean(BSFDeclaredBean bean) throws BSFException {
      try {
          TclObject tclBean = ReflectObject.newInstance( interp, bean.type,
                                                         bean.bean );
          interp.unsetVar( bean.name, 0 );
      } catch( Exception e ) {
	  throw new BSFException (BSFException.REASON_EXECUTION_ERROR,
	                          "error while undeclaring bean '" + bean.name +
				  "': " + e.toString(), e);
      }
  }

  /*
   * This is used by an application to evaluate a string containing
   * some expression.
   */
  public Object eval (String source, int lineNo, int columnNo, 
		      Object oscript) throws BSFException {
    try {
        interp.eval( oscript.toString() );

	TclObject result      = interp.getResult();
	Object    internalRep = result.getInternalRep();

	// if the object has a corresponding Java type, unwrap it
	if (internalRep instanceof ReflectObject)
	  return ReflectObject.get(interp,result);
	if (internalRep instanceof TclString)
	  return result.toString();
	if (internalRep instanceof TclDouble)
	  return new Double(TclDouble.get(interp,result));
	if (internalRep instanceof TclInteger)
	  return new Integer(TclInteger.get(interp,result));

	return result;
    }
    catch( Exception e ) {
	throw new BSFException (BSFException.REASON_EXECUTION_ERROR,
	                        "error while eval'ing TCL expression: " +
				e.toString(), e);
    }
  }

  public Object call (Object object, String method, Object[] args)
                                                        throws BSFException {
    StringBuffer sb = new StringBuffer (300);

    System.out.println("in call");
    sb.append (method);
    if (args != null) {
      sb.append (" ");
      for (int i = 0; i < args.length; i++) {
        sb.append (args[i].toString ());
      }
    }
    return eval ("<unknown>", -1, -1, sb.toString ());
  }

}
